package org.tbasket.api

import org.tbasket.api.compute.APIRequestHandler
import zio._
import zio.http.ServerConfig.LeakDetectionLevel
import zio.http._

import scala.collection.mutable

class Endpoint(hostname: String, port: Int, rootPath: Option[String]) extends ZIOAppDefault {


    private val Root: Path = rootPath.map(Path.decode).getOrElse(Path.root)
    private val handlers   = mutable.HashMap.empty[String, APIRequestHandler]

    def bind(path: String)(handler: APIRequestHandler): Unit = {
        handlers.put(path, handler)
    }

    private val app = Http.collectZIO[Request] {
        case r@_ -> Root / path if handlers.contains(path) =>
            handlers(path).handle.provideEnvironment(ZEnvironment(r))
        case _ -> path                                     =>
            ZIO.fail(new APIException(s"Unable to find a handler for page '$path'"))
    }
    val run = {
        val config = ServerConfig.default
                .port(port)
                .leakDetection(LeakDetectionLevel.PARANOID)

        val configLayer = ServerConfig.live(config)
        (Server.install(app).flatMap { port =>
            Console.printLine(s"Listening API entries on $hostname:$port$rootPath")
        } *> ZIO.never).provide(configLayer, Server.live)
    }
}
